#!/bin/bash
#
# author           : George Vlahavas (vlahavas~AT~gmail~DOT~com, gapan@zenwalk forums)
# project web page : http://biscuits.jwhitham.org.uk/LightYears/
# source           : http://biscuits.jwhitham.org.uk/LightYears/LightYears-1.2a.zip

# Package name
NAME="LightYears"
# Package version
VERSION="1.2a"
# Source code URL
SOURCE="http://biscuits.jwhitham.org.uk/LightYears/LightYears-$VERSION.zip"
# ./configure options
CONFIGOPTIONS="--prefix=/usr"
# Files that should be placed in /usr/doc/package-version/
DOCS="LICENSE.txt README.txt about.html hints.html install.html instructions.html story.html"

CWD=`pwd`
PKG="$CWD/$NAME-install"
ARCH="noarch"
BUILD="48.1"

rm -rf $PKG
mkdir -p $PKG

# Create package install directory
mkdir $PKG/install

# Create the slack-desc file
cat > $PKG/install/slack-desc << END
$NAME: lightyears - a single-player real-time strategy game
$NAME:
$NAME: "20,000 Light Years Into Space" was written for Pyweek (March 2006)
$NAME: by Jack Whitham. It is a single-player real-time strategy game
$NAME: with a science-fiction theme.
$NAME: 
$NAME:
$NAME:
$NAME:
$NAME:
$NAME:
END

# Check to see if source is present. If it's not, download it
rm -rf $NAME
if [ -f $NAME-$VERSION.zip ]; then
	echo "Source file already here, no need to download";
else
 	echo "Downloading source"
	wget -c $SOURCE
fi

# Check to see if icons are present. If it's not, download it
if [ -f icons.tar.gz ]; then
	echo "Icon file already here, no need to download";
else
 	echo "Downloading icon file"
	wget -c http://users.zenwalk.org/user-accounts/gapan/$NAME/icons.tar.gz
fi

# Unpack source
unzip $NAME-$VERSION.zip
tar xvzf icons.tar.gz

# Create the desktop entry
mkdir -p $PKG/usr/share/applications
cat > $PKG/usr/share/applications/$NAME.desktop <<EOF
[Desktop Entry]
Encoding=UTF-8
Name=20000 light years in space
Exec=lightyears
Terminal=false
Type=Application
Categories=Game;StrategyGame;
Icon=LightYears
EOF

# Install on temporary directory
cd $NAME
mkdir -p $PKG/usr/share/$NAME
cp -a LightYears.py code data $PKG/usr/share/$NAME

# Create a launch script
mkdir -p $PKG/usr/bin
cat > $PKG/usr/bin/lightyears <<EOF
#!/bin/sh
cd /usr/share/$NAME
python ./LightYears.py
EOF
chmod a+x $PKG/usr/bin/lightyears

# Copy icons to the right place
ICONSIZES="64 48 32 24 16"
for i in $ICONSIZES; do
	mkdir -p $PKG/usr/share/icons/hicolor/$i\x$i/apps;
	mv $CWD/$NAME-$i.png $PKG/usr/share/icons/hicolor/$i\x$i/apps/$NAME.png;
done;

# Copy Docs
mkdir -p $PKG/usr/doc/$NAME-$VERSION
cd $CWD/$NAME
cp -a $DOCS $PKG/usr/doc/$NAME-$VERSION

# Add the build-file in the package
mkdir -p $PKG/usr/src/$NAME-$VERSION
cp "$CWD/build-$NAME.sh" "$PKG/usr/src/$NAME-$VERSION/build-$NAME.sh"
chmod a+x "$PKG/usr/src/$NAME-$VERSION/build-$NAME.sh"

# Make sure ownerships and permissions are sane
cd $PKG
chown -R root:root .
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;
find . -perm 600 -exec chmod 644 {} \;
find . -perm 444 -exec chmod 644 {} \;
find . -perm 400 -exec chmod 644 {} \;
find . -perm 440 -exec chmod 644 {} \;
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 511 -exec chmod 755 {} \;
find . -perm 711 -exec chmod 755 {} \;
find . -perm 555 -exec chmod 755 {} \;

# Make the package
makepkg -l y -c n $CWD/$NAME-$VERSION-$ARCH-$BUILD.tgz

# Calculate md5sum
cd $CWD/
md5sum $NAME-$VERSION-$ARCH-$BUILD.tgz > $NAME-$VERSION-$ARCH-$BUILD.md5

# Remove source-code and temporary install directories
# since they are not needed anymore
rm -rf $CWD/$NAME
rm -rf $PKG
